/*
    Physics Simulation

    (C) 2007 Jason Hunt
    nulluser@gmail.com

    File: math3d.cpp
*/


#include "core.h"



/* Set the value of a graphics vertex from a face point */
void vertex::set( face_mass_type &f )
{
    x = f.mass->world_pos.x; 
    y = f.mass->world_pos.y; 
    z = f.mass->world_pos.z; 
    
    u = f.tx_x; 
    v = f.tx_y;
}
/* End of set_vertex */



/* / operator for vector */
vector operator/ ( vector p, double s )
{
    if (fabs(s) < epsilon) return(vector());

    return vector(p.x / s, p.y / s, p.z / s);
}
/* End of / operator for vector */


/* * operator for vector */
vector operator* ( double s, vector p )
{
    return vector(s * p.x, s * p.y, s * p.z);
}
/* End of * operator for vector */


/* * operator for vector */
vector operator* ( vector p, double s )
{
    return(s * p);
}
/* End of * operator for vector */



/* + operator for vector */
vector operator+ ( vector p1, vector p2 )
{
    return vector(p1.x + p2.x, p1.y + p2.y, p1.z + p2.z);
}
/* End of + operator for vector */



/* - operator for vector */
vector operator- ( vector p1, vector p2 )
{
    return vector(p1.x - p2.x, p1.y - p2.y, p1.z - p2.z);
}
/* End of - operator for vector */


/* += operator for vector */
vector operator+= ( vector &p1, vector p2 )
{
    p1 = p1 + p2;
    return(p1);
}
/* End of += operator for vector */


/* -= operator for vector */
vector operator-= ( vector &p1, vector p2 )
{
    p1 = p1 - p2;
    return(p1);
}
/* End of -= operator for vector */


/* Compute dot product */
double dot( vector p1, vector p2 )
{
    return(p1.x * p2.x + p1.y * p2.y + p1.z * p2.z);
}
/* End of dot product */



/* Compute cross product */
vector cross( vector p1, vector p2 )
{
    return vector(p1.y * p2.z - p1.z * p2.y,
                  p1.z * p2.x - p1.x * p2.z,
                  p1.x * p2.y - p1.y * p2.x);
}
/* End of cross product */


/* Compute magnitude of a vector */
double vector::mag( void )
{
    return(sqrt(x*x + y*y + z*z));
}
/* End of mag */


vector vector::norm(void)
{
    double m = this->mag();
   
    if (m < epsilon) m = epsilon;   
 
     return(*this / m);   
}

/* Normalize a vector */
void vector::normalize( void )
{
    double m = this->mag();
   
    if (m < epsilon) m = epsilon;
    
    *this = *this / m;
}
/* End of Normalize vector */



// State Operations


/* * operator for state */
state operator* ( double s, state p )
{
    return state(s * p.pos, s * p.vel);
}
/* End of * operator for state */


/* / operator for state */
state operator/ ( state p, double s )
{
    if (fabs(s) < epsilon) return(state());

    return(state(p.pos / s, p.vel / s));
}
/* End of / operator for state */


/* + operator for state */
state operator+ ( state p1, state p2 )
{
    return state(p1.pos + p2.pos, p1.vel + p2.vel);
}
/* End of + operator for state */


/* - operator for state */
state operator- ( state p1, state p2 )
{
    return state(p1.pos - p2.pos, p1.vel - p2.vel);
}
/* End of - operator for state */


/* += operator for state */
state operator+= ( state &s1, state s2 )
{
    s1 = s1 + s2;
    return(s1);
}
/* End of += operator for state */


/* -= operator for state */
state operator-= ( state &s1, state s2 )
{
    s1 = s1 - s2;
    return(s1);
}
/* End of -= operator for state */



/* Quaternion + operator */
quaternion operator+ ( quaternion q1, quaternion q2 )
{
    return quaternion(q1.v + q2.v, q1.w + q2.w);
}
/* End of quaternion + operator */


/* Quaternion += operator */
quaternion operator+= ( quaternion &q1, quaternion q2 )
{
    q1 = q1 + q2;
    return(q1);
}
/* End of quaternion += operator */


/* Quaternion - operator */
quaternion operator- ( quaternion q1, quaternion q2 )
{
    return quaternion(q1.v - q2.v, q1.w - q2.w);
}
/* End of quaternion - operator */


/* Quaternion -= operator */
quaternion operator-= ( quaternion &q1, quaternion q2 )
{
    q1 = q1 - q2;
    return(q1);
}
/* End of quaternion -= operator */


/* Quaternion * operator */
quaternion operator* ( quaternion q1, quaternion q2 )
{
    return quaternion(q1.w * q2.v + q2.w * q1.v + cross(q1.v, q2.v),
                      q1.w * q2.w - dot(q1.v, q2.v));
}
/* End of quaternion * operator */


/* Quaternion * operator */
quaternion operator* ( quaternion q, vector v )
{
    return quaternion(q.w*v.x + q.v.y*v.z - q.v.z*v.y,
                      q.w*v.y + q.v.z*v.x - q.v.x*v.z,
                      q.w*v.z + q.v.x*v.y - q.v.y*v.x,
                      -(q.v.x*v.x + q.v.y*v.y + q.v.z*v.z));
}
/* End of quaternion * operator */


/* Quaternion * operator */
quaternion operator* ( vector v, quaternion q )
{
    return quaternion(q.w*v.x + q.v.z*v.y - q.v.y*v.z,
                      q.w*v.y + q.v.x*v.z - q.v.z*v.x,
                      q.w*v.z + q.v.y*v.x - q.v.x*v.y,
                      -(q.v.x*v.x + q.v.y*v.y + q.v.z*v.z));
}
/* End of quaternion * operator */



/* Quaternion * operator */
quaternion operator* ( quaternion q, double s )
{
    return quaternion(q.v * s, q.w * s);
}
/* End of quaternion * operator */


/* Quaternion / operator */
quaternion operator/ ( quaternion q, double s )
{
    if (fabs(s) < epsilon) return quaternion();

    return quaternion(q.v / s, q.w / s);
}
/* End of quaternion / operator */


/* Return the conjugate of a quaternion */
quaternion quaternion::conjugate( void )
{
    return quaternion(-this->v.x, -this->v.y, -this->v.z, this->w);
}
/* End of quaternion::conjugate* /


/* Get a quaternion from axis angles */
quaternion quaternion_from_angles( vector ang )
{    
    // Compute unit orientations about each axis
    quaternion q_x = quaternion(vector(sin(ang.x/2.0), 0, 0), cos(ang.x/2.0));
    quaternion q_y = quaternion(vector(0, sin(ang.y/2.0), 0), cos(ang.y/2.0));
    quaternion q_z = quaternion(vector(0, 0, sin(ang.z/2.0)), cos(ang.z/2.0));
    
    return(q_z * q_y * q_x);
}
/* End of quaternion + operator */


/* Compute magnitude of a quaternion */
double quaternion::mag( void )
{
    return sqrt(v.x * v.x + v.y * v.y + v.z * v.z + w * w);
}
/* End of quaternion::mag */


/* Normalize a quaternion */
void quaternion::normalize( void )
{
    double m = this->mag();
    
    if (m < epsilon) m = epsilon;
    
    *this = *this / m;
}
/* End of quaternion::normalize */


/* Rotate a vector around a quaternion */
void quaternion_rotate_vector( quaternion q, vector &v )
{
    quaternion r = q*v*q.conjugate();

    v = r.v;
}
/* End of quaternion_rotate_vector*/






double length( point2d p )
{
    return sqrt(p.x * p.x + p.y * p.y);
}


void normalize(point2d p)
{
    double l = length(p);
    
    if (l < epsilon) l = epsilon;
    
    p = p / l;
}



point2d get_ortho_unit(point2d p)
{
    point2d r;
    r.x = -p.y;
    r.y = p.x;
    
    normalize(r);
    return(r);
}


// Returns direction of cross product
double cross(point2d p1, point2d p2)
{
    return p2.x * p1.y - p1.x * p2.y;    
}



double dot ( point2d p1, point2d p2)
{
return (p1.x * p2.x + p1.y * p2.y)  ;  
    
}

/* Turn two cords into a point */
 point2d get_point( double x, double y)
{
    point2d p;
    p.x = x; p.y=y;
    return(p);
}
/* End of get point */


/* - operator for points */
point2d operator/(point2d p, double s)
{
    point2d r;

    if (fabs(s) < 0.00001) return(get_point(0,0));
    r.x = p.x/s;
    r.y = p.y/s;
    return(r);
}

/* - operator for points */
point2d operator* ( double s, point2d p)
{
    point2d r;
    r.x = s * p.x;
    r.y = s * p.y;
    return(r);
}

/* - operator for points */
point2d operator* ( point2d p, double s)
{
    return(s * p);
}

/* - operator for points */
point2d operator- ( point2d p1, point2d p2)
{
    point2d r;
    r.x = p1.x - p2.x;  r.y = p1.y - p2.y;
    return(r);
}

/* - operator for points */
point2d operator+ ( point2d p1, point2d p2)
{
    point2d r;
    r.x = p1.x + p2.x;   r.y = p1.y + p2.y;
    return(r);
}

/* - operator for points */
point2d operator+= ( point2d &p1, point2d p2)
{
    p1.x += p2.x; p1.y += p2.y;
    return(p1);
}

/* - operator for points */
point2d operator-= ( point2d &p1, point2d p2)
{
    p1.x -= p2.x;
    p1.y -= p2.y;
    return(p1);
}









